/*
  (c) 2002, Jun Kurabe and Hiroshi Ichiji. All rights reserved.

  Time-stamp: <2002/11/19 22:54:56 jun>

 */
// Created by Jun Kurabe

import javax.mail.Folder;
import javax.mail.MessagingException;
import javax.mail.Message;
import javax.mail.NoSuchProviderException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.URLName;

import java.util.Properties;
import java.util.Vector;

import MailProperties;
import MailFileException;
import Reference;

/**
 * Describe class <code>MailFile</code> here.
 *
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
class MailFile {
    /** Mail URL */
    public static String url = "mh:/home/jun2";

    //    public static final String url = "mh:///c|/Home/";
    //    public static final String url = "mh:/Home/Mail/";

    /** */
    public static final String ROOT = "Mail";
    //    static final String root = "inbox";

    /** 
     * Mail File Object
     * @param ref
     * @exception MailFileException
     */
    public MailFile (Reference ref, MailProperties mp) 
	throws MailFileException {
	this.ref = ref;
	this.mp = mp;
    }

    /** 
     * open the mail file and make connection to messaging store
     * @exception MailFileException 
     */
    public void open() 
	throws MailFileException {
	Properties props = System.getProperties();

	session = Session.getDefaultInstance(props, null);
	//	session.setDebug(true);

	String tmpUrl = mp.getMailPath();
	if (tmpUrl != null) url = tmpUrl;

	urln = new URLName(url);
	//	System.err.println("urln = " + urln);
	try {
	    store = session.getStore(urln);
	    store.connect();
	} catch (NoSuchProviderException e) {
	    e.printStackTrace();
	    throw new MailFileException();
	} catch (MessagingException e) {
	    e.printStackTrace();
	    throw new MailFileException();
	}
    }

    /**
     * close mail file
     * @exception MailFileException if an error occurs
     */
    public void close() 
	throws MailFileException {
	try {
	    store.close();
	} catch ( javax.mail.MessagingException e) {
	    e.printStackTrace();
	    throw new MailFileException();
	}
    }

    /**
     * List sub folders
     * @param fn a <code>String</code> value
     * @return Vector
     * @exception MailFileException
     */
    public Vector getSubFolders(String fn) 
	throws MailFileException {
	try {
	    //	    System.err.println(store.toString());
	    // ե
	    Folder rf = store.getFolder(fn);

	    Vector anArray = new Vector();
	    
	    getSubFolderMain(rf, 0, anArray);
	    //	    System.err.println("Messages = " + anArray.size());
	    return anArray;
	} catch (javax.mail.MessagingException e) {
	    e.printStackTrace();
	    throw new MailFileException();
	}
    }

    /** 
     * Get the subfolders recursively
     * @param f parent folder
     * @param level level of folder ( start is "0" )
     * @param folders Mail Folder objects store in the vector object
     * @exception MessagingException
     */
    private void getSubFolderMain(Folder f, int level, Vector folders) 
        throws MessagingException {
        level++;
	// եͤɲ
	folders.addElement(new MailFolder(f.getName(), f.getFullName(), level));
	// եˤå
	System.out.println("Folder:" + f.getFullName());
	f.open(Folder.READ_ONLY);
	int cntMsg = f.getMessageCount();

	for (int j = 0; j < cntMsg; ) {
	    Message msg = f.getMessage(++j);
	    ref.getReferences(msg);	     
	    msg = null;
	} // end of for (int j = 0; j < cntMsg; j++)

	/*
	  Message[] msgs = f.getMessages();
	  for ( int j =0; j<msgs.length; j++) {
	    //  msgMap.put(hdrs[0], new Integer(i));
	    ref.getReferences(msgs[j]);
	    }
	*/

	//	System.err.println("Ref size = " + ref.size());

	// եˤ륵֥ե
        Folder[] subFolders = f.listSubscribed();

	// ꥫ֤˥֥ե롣
        for (int i =0; i<subFolders.length; i++) {
            getSubFolderMain(subFolders[i], level, folders);
        }
	
	subFolders = null;
    }

    /** 
     * եˤåФ
     * @param folderName
     * @return Message 
     * @exception MailFileException
     */
    public Message[] getMessages(String folderName) 
	throws MailFileException  {
	try {
	    Folder rf = store.getFolder(folderName);	
	    rf.open(Folder.READ_ONLY);
	    return rf.getMessages();
	} catch ( javax.mail.MessagingException e ) {
	    e.printStackTrace();
	    throw new MailFileException();
	}
    }

    /** 
     * եˤ륵֥եФ
     * @param folderName
     * @return Message 
     * @exception MailFileException
     */
    public Folder getFolder(String folderName) 
	throws MailFileException  {
	try {
	    Folder rf = store.getFolder(folderName);	
	    rf.open(Folder.READ_ONLY);
	    return rf;
	} catch ( javax.mail.MessagingException e ) {
	    e.printStackTrace();
	    throw new MailFileException();
	}
    }

    /** */
    private Session session = null;
    /** URL Name*/
    private URLName urln = null;
    /** */
    private Store store = null;
    /** */
    private Reference ref = null;
    /** */
    private MailProperties mp = null;
}
