/*
  (c) 2002, Jun Kurabe and Hiroshi Ichiji. All rights reserved.
  
  Time-stamp: <2003/02/11 18:02:48 jun>
  
 */
// Created by Jun Kurabe
// $Date$
// $Revision$

import java.awt.Frame;
import java.util.HashMap;
import java.util.Vector;
import java.util.Enumeration;

import EDU.auburn.VGJ.graph.Node;
import EDU.auburn.VGJ.graph.Edge;
import EDU.auburn.VGJ.graph.Graph;
import EDU.auburn.VGJ.gui.GraphCanvas;

import EDU.auburn.VGJ.gui.GraphCanvasInterface;
import EDU.auburn.VGJ.gui.GraphCanvas;
import EDU.auburn.VGJ.gui.GraphWindow;
import EDU.auburn.VGJ.gui.NodeBlink;
import EDU.auburn.VGJ.util.*;
import EDU.auburn.VGJ.algorithm.tree.TreeAlgorithm;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Address;
import javax.mail.internet.MimeUtility;
import javax.mail.internet.ParseException;
import java.awt.*;
import java.awt.Graphics.*;
import java.awt.Color;

import MailGraph;
import MailDraw;
import MailFieldConvert;
import PrefaceDebug;
import Reference;

/**
 * Describe class <code>MailGraphCanvas</code> here.
 *
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
class MailGraphCanvas 
    extends GraphCanvas 
    implements MailDraw {
    
    /** */
    public static final String PROP_MSGID = "MsgId";
    /** */
    public static final String PROP_CONTENT = "Content";

    /**
     * Creates a new <code>MailGraphCanvas</code> instance.
     *
     * @param graph_in a <code>Graph</code> value
     * @param frame_in a <code>Frame</code> value
     */
    public MailGraphCanvas (Graph graph_in, Frame frame_in) {
	super(graph_in, frame_in);
	Node.defaults.setBoundingBox(20, 20, 20);
	tree = new TreeAlgorithm('d');
	nodeBlink = new NodeBlink(this);

	vSpacing /= 2.0;
	hSpacing /= 2.0;

	gap_v = vSpacing;
	gap_h = hSpacing;
    }

    /**
     * β
     * դΡɤκ
     *
     */
    public void clear() {
	unselectItems();
	((MailGraph)graph_).clear();
	passed = new HashMap();
	color = new HashMap();
	edgeColor = new HashMap();
	colorMgr = new ColorManager();
	edgeColorMgr = new ColorManager();
    }

    /**
     * Describe <code>addCanvasAdapter</code> method here.
     *
     * @param cwi a <code>GraphCanvasInterface</code> value
     */
    public void addCanvasAdapter(GraphCanvasInterface cwi) {
	addCalledObject(cwi);
    }

    /**
     * ֥Ȥ󤹤
     *
     */
    private void applyAlgorithm() {
	graph_.removeGroups();  // Remove group nodes.
	graph_.pack();  // Pack the indexes.
               
	String msg;
	setWireframe(true);
	msg = tree.compute(graph_, this);
	setWireframe(false);
	    
	/*	if(msg != null)
	    new MessageDialog(this, "Message", msg, true);
	*/     
	this.update(true);
    }

    /**
     * Describe <code>draw</code> method here.
     *
     * @param msgId
     * @param ref
     */
    public void draw(String msgId, Reference ref) {

	class DMG 
	    extends RIImpl {
	    public DMG(String msgId, Reference ref, MailGraphWindow mgw) {
		this.ref = ref;
		this.mgw = mgw;
		passed.put(msgId, msgId);
	    }

	    /** */
	    public boolean followDoIt(String orgId, String childId) {
		try {
		    if ( passed.get(childId) == null ) {
			mgw.addNode(childId, ref.getMsg(childId));
			passed.put(childId, childId);
		    }
		    mgw.addEdge(orgId, childId);
		    System.err.println(orgId + "-->" + childId);
		} catch (MailFileException e) {
		    e.printStackTrace();
		}
		return false;
	    }

	    /**  */
	    private HashMap passed = null;
	    /** */
	    private Reference ref = null;
	    /** */
	    private MailGraphWindow mgw = null;
	}

	xref = ref;

	String top = ref.draw(msgId, this);
	//gw.draw(top);
	this.applyAlgorithm();
    }

    /**
     * Describe <code>addNode</code> method here.
     *
     * @param id
     * @param msg
     */
    public void addNode(String id, Message msg) {
	int fId = graph_.insertNode();
	Node node = graph_.getNodeFromIndex(fId);
	node.setBoundingBox(300.0d, 40.0d);
	node.setShape(Node.RECTANGLE);

	try {
	    String label = MailFieldConvert.convertSubject(msg);
	    String aFrom = MailFieldConvert.convertFrom(msg);

	    int ip = aFrom.indexOf("<");
	    if (ip != -1) {
		aFrom = aFrom.substring(0,ip);
	    } // end of if (ip != -1)
	    
	    Color aColor = (Color)color.get(aFrom);
	    if (aColor == null ) {
		aColor = colorMgr.getColor();
		color.put(aFrom, aColor);
	    }
	    node.setColor(aColor);

	    int atP = aFrom.indexOf("@");
	    String aFromAt = aFrom;
	    if (atP != -1) {
		aFromAt = aFrom.substring(0,atP);
	    } // end of if (atP != -1)
	    
	    node.setLabel(aFromAt);
	    node.setLabelPosition("CENTER");
	    //	    Graphics gr = gw.getGraphics();
	    Graphics gr = getGraphics();
	    if ( gr == null ) 
		node.setBoundingBox(24.0d * label.length(), 26.0d);
	    else {
		FontMetrics fm = gr.getFontMetrics();

		int wl = fm.stringWidth(aFromAt);

		//		node.setBoundingBox(wl, fm.getHeight()*3.0d); // 3ԤʤΤǡ30Ǥ
		node.setBoundingBox(wl, fm.getHeight()*1.0d); // 3ԤʤΤǡ30Ǥ
		//	    node.setBoundingBox(getStringWidth(label), 26.0d);
	    }
	    node.data_.put(PROP_CONTENT, MailFieldConvert.getBodyText(msg));
	    //	    node.data_.put(PROP_MSGID,msg.getHeader("Message-ID")[0]);
	    node.data_.put(PROP_MSGID,MailFieldConvert.getMsgId(msg));
	} catch (MailFileException me) {
	    me.printStackTrace();
	}

	PrefaceDebug.println("id =" + id,this);
	passed.put(id, new Integer(fId));
    }

    /** */
    static private final double as = 8.0d;
    /** */
    static private final double ds = 16.0d;

    /**
     * Add Edge between nodes
     * @param formId
     * @param toId
     */
    public void addEdge(String fromId, String toId) {
	Integer fId = (Integer)passed.get(fromId);
	Integer tId = (Integer)passed.get(toId);

	// IDnullʤ֤
	if (fId == null || tId == null ) return;

	Edge edge = graph_.insertEdge(fId.intValue(), tId.intValue());

	Message msg = null;
	try {
	    msg = xref.getMsg(toId);	     
	    String label = MailFieldConvert.getBareSubject(msg);

	    Color c = (Color)edgeColor.get(label);
	    if (c == null) {
		c = edgeColorMgr.getColor();
		edgeColor.put(label, c);
		//System.err.println("Color:" + c);
	    } // end of if (c == null)
	    edge.setColor(c);
	    // Label
	    //	    edge.setLabel(label);
	} catch (MailFileException e) {
	    
	} // end of try-catch
	
	PrefaceDebug.println("fromId =" + fromId + " toId = " + toId,this);
    }

    /**
     * Add Edge between mails
     * @param graph
     * @param nodes
     * @param from
     * @param to
     */
    public void addEdge(MailGraph graph, Nodes nodes, int from, int to) {
	int fromId = nodes.getFromIdx(from);
	int toId = nodes.getFromIdx(to);

	graph.insertEdge(fromId, toId);
    }

    /**
     * ΡɤΥ˥졼֤
     *
     * @return an <code>Enumeration</code> value
     */
    public Enumeration getNodes() {
	return ((MailGraph)graph_).getNodes();
    }

    /**
     * ǤΡɤID֤
     *
     * @return a <code>NodeBlink</code> value
     */
    public NodeBlink getNodeBlink() {
	return nodeBlink;
    }

    /*    public void setCanvas(GraphCanvas graphCanvas) {
	graphCanvas_ = graphCanvas;
    }

    */
    /**
     * 򤵤ƤΡɤΥåID롣
     *
     * @return String
     */
    public String getSelectedMsgId() {
	Node n = getSelectedNode();
	if (n == null ) return null;

	return (String)n.data_.get("MsgId");
    }

    public void setScale(double scale) {
	super.setScale(scale);
	vSpacing = gap_v * scale;
	hSpacing = gap_h * scale;
    }

    /**
     * Describe <code>getSelectedMsgId</code> method here.
     *
     * @return a <code>String</code> value
     */
    //    public String getSelectedMsgId() {
    //	return gw.getSelectedMsgId();
    //    }

    /**
     * Describe <code>unselectItems</code> method here.
     *
     */
    //    public void unselectItems() {
    //	gw.unselectItems();
    //    }

    /** */
    //    private MailGraph graph = null;
    /** */
    //    private MailGraphWindow gw = null;
    /** */

    private HashMap passed = null;

    /** */
    private HashMap color = null;
    /** */
    private HashMap edgeColor = null;
    /** */
    private ColorManager colorMgr = null;
    /** */
    private ColorManager edgeColorMgr = null;

    /** */
    private NodeBlink nodeBlink = null;

    /** */
    static private Reference xref = null;

    /** */
    private TreeAlgorithm tree = null;

    /** */
    private double gap_h = 1.0;
    private double gap_v = 1.0;
}
