/*
  (c) 2002, Jun Kurabe and Hiroshi Ichiji. All rights reserved.

Created by Jun Kurabe
2001/09/16
 */

//package jp.co.lmer.mail;

import java.util.HashMap;
import java.util.Vector;
import java.util.Enumeration;

import EDU.auburn.VGJ.graph.Node;
import EDU.auburn.VGJ.graph.Edge;
import EDU.auburn.VGJ.util.*;
import EDU.auburn.VGJ.gui.GraphCanvasInterface;
import EDU.auburn.VGJ.gui.GraphCanvas;
import EDU.auburn.VGJ.gui.GraphWindow;
import EDU.auburn.VGJ.gui.NodeBlink;
import EDU.auburn.VGJ.algorithm.tree.TreeAlgorithm;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Address;
import javax.mail.internet.MimeUtility;
import javax.mail.internet.ParseException;
import java.awt.*;
import java.awt.Graphics.*;
import java.awt.Color;

import MailGraph;
import MailDraw;
import MailFieldConvert;
import PrefaceDebug;
import Reference;

/**
 * @author Jun Kurabe
 */
public class MailGraphWindow
    extends GraphWindow
    implements MailDraw {

    /** */
    public static final String PROP_MSGID = "MsgId";
    public static final String PROP_CONTENT = "Content";

    /**
     * Creates a new <code>MailGraphWindow</code> instance.
     *
     * @param cwi[] a <code>GraphCanvasInterface</code> value
     */
    public MailGraphWindow() {
	super(new MailGraph());

	// TreeAlgorithm parameter: d, u, l, r
	TreeAlgorithm tree = new TreeAlgorithm('d');
	this.addAlgorithm(tree, "JKTree->d");
	this.setSize(600,500);

	nodeBlink = new NodeBlink(this.graphCanvas_);
    }

    /**
     * Describe <code>addCanvasAdapter</code> method here.
     *
     * @param cwi a <code>GraphCanvasInterface</code> value
     */
    public void addCanvasAdapter(GraphCanvasInterface cwi) {
	graphCanvas_.addCalledObject(cwi);
    }

    /**
     * Describe <code>destroy</code> method here.
     *
     */
    public void destroy() {
	//	gw.setVisible(false);
	setVisible(false);
	//	gw = null;
	//	graph = null;
    }

    /**
     * Clear HashTable and Node on Graph
     */
    public void clear() {
	unselectItems();
	((MailGraph)graph_).clear();
	passed = new HashMap();
	color = new HashMap();
	edgeColor = new HashMap();
	colorMgr = new ColorManager();
	edgeColorMgr = new ColorManager();
    }

    /**
     * Describe <code>draw</code> method here.
     *
     * @param title a <code>String</code> value
     */
    public void draw(String title) {
	this.setTitle(title);
	//	this.show();
	// 2002/11/07 
	//	this.setVisible(true);
	// End
	this.applyAlgorithm("JKTree->d");
    }

    /**
     * Describe <code>draw</code> method here.
     *
     * @param msgId
     * @param ref
     */
    public void draw(String msgId, Reference ref) {

	class DMG 
	    extends RIImpl {
	    public DMG(String msgId, Reference ref, MailGraphWindow mgw) {
		this.ref = ref;
		this.mgw = mgw;
		passed.put(msgId, msgId);
	    }

	    /** */
	    public boolean followDoIt(String orgId, String childId) {
		try {
		    if ( passed.get(childId) == null ) {
			mgw.addNode(childId, ref.getMsg(childId));
			passed.put(childId, childId);
		    }
		    mgw.addEdge(orgId, childId);
		    System.err.println(orgId + "-->" + childId);
		} catch (MailFileException e) {
		    e.printStackTrace();
		}
		return false;
	    }

	    /**  */
	    private HashMap passed = null;
	    /** */
	    private Reference ref = null;
	    /** */
	    private MailGraphWindow mgw = null;
	}

	xref = ref;

	String top = ref.draw(msgId, this);
	//gw.draw(top);
	draw(top);
    }

    /**
     * Show the Window
     */
    //    public void show() {
    //	gw.show();
    //    }
    
    /**
     * @param id
     * @param msg
     */
    public void addNode(String id, Message msg) {
	//	if ( passed.get(id) != null) return;
	int fId = graph_.insertNode();
	Node node = graph_.getNodeFromIndex(fId);
	node.setBoundingBox(300.0d, 40.0d);
	node.setShape(Node.RECTANGLE);
	//	node.setLabel(id);

	try {
	    String label = MailFieldConvert.convertSubject(msg);
	    String aFrom = MailFieldConvert.convertFrom(msg);

	    int ip = aFrom.indexOf("<");
	    if (ip != -1) {
		aFrom = aFrom.substring(0,ip);
		//		String aFrom1 = aFrom.substring(0,ip-1);
		// String aFrom2 = aFrom.substring(ip);
		//		aFrom = aFrom1 + "\\n" + aFrom2;
	    } // end of if (ip != -1)
	    
	    Color aColor = (Color)color.get(aFrom);
	    if (aColor == null ) {
		//		aColor = Color.pink;
		aColor = colorMgr.getColor();
		color.put(aFrom, aColor);
	    }
	    node.setColor(aColor);

	    //	    node.setLabel(label+"\\n"+aFrom);

	    int atP = aFrom.indexOf("@");
	    String aFromAt = aFrom;
	    if (atP != -1) {
		aFromAt = aFrom.substring(0,atP);
	    } // end of if (atP != -1)
	    
	    node.setLabel(aFromAt);
	    node.setLabelPosition("CENTER");
	    //	    Graphics gr = gw.getGraphics();
	    Graphics gr = getGraphics();
	    if ( gr == null ) 
		node.setBoundingBox(24.0d * label.length(), 26.0d);
	    else {
		FontMetrics fm = gr.getFontMetrics();
		/*
		  int wl = fm.stringWidth(label);
		  int wf1 = fm.stringWidth(aFrom1);
		  int wf2 = fm.stringWidth(aFrom2);
		  if ( wf1 > wl ) wl = wf1;
		  if ( wf2 > wl ) wl = wf2;
		*/

		int wl = fm.stringWidth(aFromAt);

		//		node.setBoundingBox(wl, fm.getHeight()*3.0d); // 3ԤʤΤǡ30Ǥ
		node.setBoundingBox(wl, fm.getHeight()*1.0d); // 3ԤʤΤǡ30Ǥ
		//	    node.setBoundingBox(getStringWidth(label), 26.0d);
	    }
	    node.data_.put(PROP_CONTENT, MailFieldConvert.getBodyText(msg));
	    //	    node.data_.put(PROP_MSGID,msg.getHeader("Message-ID")[0]);
	    node.data_.put(PROP_MSGID,MailFieldConvert.getMsgId(msg));
	} catch (MailFileException me) {
	    me.printStackTrace();
	}

	PrefaceDebug.println("id =" + id,this);
	passed.put(id, new Integer(fId));
    }

    /** */
    static private final double as = 8.0d;
    /** */
    static private final double ds = 16.0d;

    /**
     * Add Edge between nodes
     * @param formId
     * @param toId
     */
    public void addEdge(String fromId, String toId) {
	//	System.out.println("Called" + fromId + ":" + toId);
	Integer fId = (Integer)passed.get(fromId);
	Integer tId = (Integer)passed.get(toId);

	if (fId == null || tId == null ) return;

	Edge edge = graph_.insertEdge(fId.intValue(), tId.intValue());

	Message msg = null;
	try {
	    msg = xref.getMsg(toId);	     
	    String label = MailFieldConvert.getBareSubject(msg);

	    Color c = (Color)edgeColor.get(label);
	    if (c == null) {
		c = edgeColorMgr.getColor();
		edgeColor.put(label, c);
		System.err.println("Color:" + c);
	    } // end of if (c == null)
	    edge.setColor(c);
	    // Label
	    //	    edge.setLabel(label);
	} catch (MailFileException e) {
	    
	} // end of try-catch
	
	PrefaceDebug.println("fromId =" + fromId + " toId = " + toId,this);
    }

    /**
     * Add Edge between mails
     * @param graph
     * @param nodes
     * @param from
     * @param to
     */
    public void addEdge(MailGraph graph, Nodes nodes, int from, int to) {
	//	System.out.println("Called2:" + from + ":" + to);
	int fromId = nodes.getFromIdx(from);
	int toId = nodes.getFromIdx(to);

	graph.insertEdge(fromId, toId);
    }

    /**
     * ΡɤΥ˥졼֤
     *
     * @return an <code>Enumeration</code> value
     */
    public Enumeration getNodes() {
	return ((MailGraph)graph_).getNodes();
    }

    /**
     * Describe <code>getNodeBlink</code> method here.
     *
     * @return a <code>NodeBlink</code> value
     */
    public NodeBlink getNodeBlink() {
	return nodeBlink;
    }

    /**
     * Describe <code>setCanvas</code> method here.
     *
     * @param graphCanvas a <code>GraphCanvas</code> value
     */
    public void setCanvas(GraphCanvas graphCanvas) {
	graphCanvas_ = graphCanvas;
    }

    /**
     * Describe <code>getSelectedMsgId</code> method here.
     *
     * @return a <code>String</code> value
     */
    //    public String getSelectedMsgId() {
    //	return gw.getSelectedMsgId();
    //    }

    /**
     * Describe <code>unselectItems</code> method here.
     *
     */
    //    public void unselectItems() {
    //	gw.unselectItems();
    //    }

    /** */
    //    private MailGraph graph = null;
    /** */
    //    private MailGraphWindow gw = null;
    /** */
    private HashMap passed = null;

    /** */
    private HashMap color = null;
    /** */
    private HashMap edgeColor = null;
    /** */
    private ColorManager colorMgr = null;
    /** */
    private ColorManager edgeColorMgr = null;

    /** */
    private NodeBlink nodeBlink = null;

    /** */
    static private Reference xref = null;

}
	
/**
 * Describe class <code>Nodes</code> here.
 *
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
class Nodes 
    extends Vector {
    /** */
    public Nodes () {
	super();
    };

    /** 
     * @param value
     */
    public void addElement(int value) {
	super.addElement(new Integer(value));
    }

    /** 
     * @param idx
     * @return int
     */
    public int getFromIdx(int idx) {
	return ((Integer)super.elementAt(idx)).intValue();
    }
}

/**
 * 2002/01/13
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
class ColorManager {
    
    /**
     * Creates a new <code>ColorManager</code> instance.
     *
     */
    public ColorManager() {
    }

    /**
     * return a Color
     * @return Color 
     */
    public Color getColor() {
	Color rv = colors[index++];
	if ( index >= colors.length) index = 0;
	return rv;
    }

    /** */
    private int index = 0;

    /** */
    static private Color[] colors = {Color.pink, 
				     Color.lightGray, 
				     Color.cyan, 
				     Color.yellow, 
				     Color.green,
				     Color.orange};
}
