/*
  (c) 2003 Jun Kurabe and Hiroshi Ichiji. All rights reserved.

  Time-stamp: <2003/02/26 00:23:21 jun>

  Revision: $Revision$
  Date: $Date$
*/

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Set;
import java.util.Vector;

import java.lang.Math;

import AddressNodeState;
import LinkInfo;
import MailPopup;
import NodeState;
import Selection;
import SelectionListener;

/**
 * Describe class <code>VisualSearchArea</code> here.
 *
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
public class VisualSearchArea 
    extends ScrollPane {

    static final int WIDTH = 365;
    static final int HEIGHT = 365;

    //    static final int WIDTH = 740;
    //    static final int HEIGHT = 700;

    static final int MODE_MSGID = 0;
    static final int MODE_ADDRESS = 1;

    /**
     * Describe class <code>VisualSearchArea</code> here.
     *
     */
    public VisualSearchArea (Frame frame,Reference ref) {
	super(ScrollPane.SCROLLBARS_ALWAYS);
	//	    super();

	canvas = new VisualSearchCanvas(frame, ref);

	setSize(WIDTH,HEIGHT);
	setVisible(true);
	canvas.setSize(WIDTH,HEIGHT);
	canvas.setVisible(true);
	add(canvas);
    }

    /**
     * Describe <code>clearSpace</code> method here.
     *
     */
    public void clearSpace() {
	canvas.clearSpace();
    }

    /**
     * Describe <code>addNode</code> method here.
     *
     */
    public void addNode(int orders[], String msgId) {
	canvas.addNode(orders, msgId);
    }

    /**
     * Describe <code>addNode</code> method here.
     *
     * @param from a <code>String</code> value
     * @param counts[] a <code>CountByAddress</code> value
     */
    public void addNode(String from, HashMap linkInfoMap) {
	canvas.addNode(from, linkInfoMap);
    }

    /**
     * Describe <code>setState</code> method here.
     *
     * @param msgId a <code>String</code> value
     * @param state an <code>int</code> value
     */
    public void setState(String msgId, int state) {
	canvas.setState(msgId, state);
    }

    /**
     * Describe <code>setMode</code> method here.
     *
     * @param mode an <code>int</code> value
     */
    public void setMode(int mode) {
	canvas.setMode(mode);
    }

    /**
     * Describe <code>getCanvas</code> method here.
     *
     * @return a <code>VisualSearchCanvas</code> value
     */
    public VisualSearchCanvas getCanvas() {
	return canvas;
    }

    /** */
    private VisualSearchCanvas canvas = null;
}

/**
 * Describe class <code>VisualSearchCanvas</code> here.
 *
 * @author <a href="mailto:jun-krb@mars.dti.ne.jp">Jun Kurabe</a>
 * @version 1.0
 */
class VisualSearchCanvas 
    extends Canvas 
    implements MouseListener, MouseMotionListener, SelectionListener {

    /**
     * Describe class <code>Pos</code> here.
     *
     */
    class Pos {
	/**
	 * Creates a new <code>Pos</code> instance.
	 *
	 * @param x an <code>int</code> value
	 * @param y an <code>int</code> value
	 */
	public Pos (int x, int y) {
	    this.x = x;
	    this.y = y;
	}

	/**
	 * Describe <code>getX</code> method here.
	 *
	 * @return an <code>int</code> value
	 */
	public int getX() {
	    return x;
	}

	/**
	 * Describe <code>getY</code> method here.
	 *
	 * @return an <code>int</code> value
	 */
	public int getY() {
	    return y;
	}

	int x = 0;
	int y = 0;
    }

    // size of Area
    static final int AREA_WIDTH = 729; // 729 + 2
    static final int AREA_HEIGHT = 729; // 729 + 2

    // place data
    static int positions[] = {0,1,3,5,7,0,2,6,8,4};

    /** */
    static final double LEVELS = 6; // 729 = 3^6

    /**
     * Creates a new <code>VisualSearchCanvas</code> instance.
     *
     */
    public VisualSearchCanvas(Frame frame,Reference ref) {
	super();
	//	setSize(AREA_WIDTH+1, AREA_HEIGHT+1);
	setSize(WIDTH+1, HEIGHT+1);

	position = new HashMap();
	distanceMap = new HashMap();

	addMouseListener(this);
	addMouseMotionListener(this);
	Selection.addListener(this);

	// ᥤξɽѥݥåץå
	mpu = new MailPopup(frame, ref);
    }

    /**
     * Describe <code>setMode</code> method here.
     *
     * @param mode an <code>int</code> value
     */
    public void setMode(int mode) {
	this.mode = mode;
	switch (mode) {
	case VisualSearchArea.MODE_ADDRESS:
	    setSelectedAddress(null);
	    break;
	default:
	    break;
	} // end of switch (mode)
    }

    static final int BOXWIDTH = 4;
    static final int BOXHEIGHT = 4;

    static final int HALF_BOXWIDTH = BOXWIDTH/2;
    static final int HALF_BOXHEIGHT = BOXHEIGHT/2;

    /**
     * Describe <code>paint</code> method here.
     *
     * @param g a <code>Graphics</code> value
     */

    public void paint(Graphics g) {
	// System.err.println("Paint Called mode=" + mode);
	switch (mode) {
	case VisualSearchArea.MODE_MSGID:
	    paintMsgId(g);
	    break;
	case VisualSearchArea.MODE_ADDRESS:
	    paintAddress(g, mode);
	    break;
	    
	default:
	    paintMsgId(g);	    
	    break;
	} // end of switch (mode)
    }

    /**
     * Describe <code>paintMsgId</code> method here.
     *
     * @param g a <code>Graphics</code> value
     */
    private void paintMsgId(Graphics g) {
	g.setColor(Color.red);

	for (int x = 0; x<space.length; x++) 
	    for (int y = 0; y < space[x].length; y++) {
		if (space[x][y] == null) continue;
		
		switch (space[x][y].getState()) {
		case NodeState.MARKED:
		    //		    System.err.println("fillOver called");
		    g.setColor(Color.red);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);		    
		    break;
		case NodeState.TEMPORARY_MARKED:
		    g.setColor(Color.green);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);		    
		    break;
		case NodeState.SELECTED:
		    g.setColor(Color.blue);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);		    
		    break;
		    
		case NodeState.PASSED:
		    //		    g.setColor(Color.black);
		    //		    g.fillOval(x,y,BOXWIDTH,BOXHEIGHT);	    
		    break;
		    
		default:
		    break;
		} // end of switch (space[x][y].getState())

	    } // end of for (int y = 0; x < space.length; y++)
	
	//	System.err.println("paint called");
    }

    /**
     * Describe <code>paintAddress</code> method here.
     *
     * @param g a <code>Graphics</code> value
     */
    private void paintAddress(Graphics g, int mode) {
	g.setColor(Color.red);
	for (int x = 0; x<space.length; x++) 
	    for (int y = 0; y < space[x].length; y++) {
		if (space[x][y] == null) continue;
		
		switch (space[x][y].getState()) {
		case NodeState.MARKED: {
		    
		    LinkInfo li  = null;
		    try {
			li = ((AddressNodeState)space[x][y]).getLinkInfo();
		    
			g.setColor(Color.green);
			Iterator addrs = li.getAddresses();
			while (addrs.hasNext()) {
			    String addr = (String)addrs.next();
			    Pos pos = (Pos)position.get(addr);
			    drawArrow(g,x,y, addr, pos, li.getCount(addr));
			}
		    } catch (NullPointerException e) {
		    } // end of try-catch
		    g.setColor(Color.red);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);   
		    break;
		}
		case NodeState.TEMPORARY_MARKED:
		    g.setColor(Color.green);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);		    
		    // drawArrow(g,x,y, (AddressNodeState)space[x][y]);
		    g.setColor(Color.black);
		    break;
		case NodeState.SELECTED:
		    g.setColor(Color.green);
		    LinkInfo li  = null;
		    try {
			li = ((AddressNodeState)space[x][y]).getLinkInfo();
		    
			g.setColor(Color.green);
			Iterator addrs = li.getAddresses();
			while (addrs.hasNext()) {
			    String addr = (String)addrs.next();
			    Pos pos = (Pos)position.get(addr);
			    drawArrow(g,x,y, addr, pos, li.getCount(addr));
			}
		    } catch (NullPointerException e) {
		    } // end of try-catch
		    g.setColor(Color.blue);
		    g.fillRect(x/2-HALF_BOXWIDTH,y/2-HALF_BOXHEIGHT,BOXWIDTH,BOXHEIGHT);		    

		    break;
		    
		case NodeState.PASSED:
		    break;
		    
		default:
		    break;
		} // end of switch (space[x][y].getState())

	    } // end of for (int y = 0; x < space.length; y++)
	
	//	System.err.println("paint called");

    }

    /**
     * Describe <code>drawArrow</code> method here.
     *
     * @param g a <code>Graphics</code> value
     * @param x an <code>int</code> value
     * @param y an <code>int</code> value
     * @param mode an <code>int</code> value
     */
    private void drawArrow(Graphics g, int x, int y, 
			   String address, Pos pos,
			   int n) {

	Graphics2D g2 = (Graphics2D) g;
	final double arrowLength = 20;

	int dx = pos.getX() - x;
	int dy = pos.getY() - y;

	double angle = 0;
	if (dx == 0) {
	    angle = Math.PI/2.0d; 
	    if (pos.getY() > y) angle = -angle;
	} 
	else {
	    angle = Math.atan((double)dy/(double)dx);
	    if (dx < 0 && dy < 0 ) {
		angle = - angle;
	    } // end of if (dx < 0 && dy < 0 )
	} // end of else

	switch (mode) {
	case VisualSearchArea.MODE_ADDRESS: {
	    double rightWingAngle = angle + Math.PI/12.0;
	    double leftWingAngle =  angle - Math.PI/12.0;

	    int rightWingX = (int)(arrowLength * Math.sin(rightWingAngle)) + x;
	    int rightWingY = (int)(arrowLength * Math.cos(rightWingAngle)) + y;
	    int leftWingX = (int)(arrowLength * Math.sin(leftWingAngle)) + x;
	    int leftWingY = (int)(arrowLength * Math.cos(leftWingAngle)) + y;

	    g2.setStroke(new BasicStroke(((float)n)/((float)cntMax)*5.0f));
	    g2.draw(new Line2D.Double((double)(x/2), (double)(y/2), 
				      (double)(pos.getX()/2),
				      (double)(pos.getY()/2)));
		//g.drawLine(x/2, y/2, pos.getX()/2, pos.getY()/2);
	}
	    break;
	default:
	    break;
	} // end of switch (mode)
    }

    /**
     * Describe <code>mouseClicked</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseClicked(MouseEvent e) {
	mouseX = e.getX();
	mouseY = e.getY();

	int nearPoint[] = isNear();
	if (nearPoint != null) {
	    switch (mode) {
	    case VisualSearchArea.MODE_MSGID: {
		String id = space[nearPoint[0]][nearPoint[1]].getId(); 
	    }
		break;
	    default:
		break;
	    } // end of switch (mode)
	} // end of if (nearPoint != null)
    }

    /**
     * Describe <code>mouseEntered</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseEntered(MouseEvent e) {
	// System.err.println("Entered");
    }

    /**
     * Describe <code>mouseExited</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseExited(MouseEvent e) {
	mouseX = -1;
	mouseY = -1;
	// System.err.println("Exited");

	// mpu.pointUp(this, mouseX,mouseY,null);
    }

    /**
     * Describe <code>mousePressed</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mousePressed(MouseEvent e) {
	mouseX = e.getX();
	mouseY = e.getY();

	if (selectedX > -1 && selectedY > -1) Selection.clear();

	int nearPoint[] = isNear();
	if (nearPoint != null) {
	    switch (mode) {
	    case VisualSearchArea.MODE_MSGID: {
		String id = space[nearPoint[0]][nearPoint[1]].getId();     
		mpu.pointDown(this, mouseX,mouseY,id);
		vX = nearPoint[0];
		vY = nearPoint[1];
	    }
		break;
	    default:
		break;
	    } // end of switch (mode)
	    
	} // end of if (nearPoint != null)

	// System.err.println("Pressed");
    }

    /**
     * Describe <code>mouseReleased</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseReleased(MouseEvent e) {
	mouseX = e.getX();
	mouseY = e.getY();
	int nearPoint[] = isNear();
	if (nearPoint == null) {
	    switch (mode) {
	    case VisualSearchArea.MODE_ADDRESS:{
		setSelectedAddress(null);
	    }
		break;
	    default:
		break;
	    }
	    return;
	}

	switch (mode) {
	case VisualSearchArea.MODE_MSGID:{
	    String id = space[nearPoint[0]][nearPoint[1]].getId();     
	    mpu.pointUp(this, mouseX,mouseY,id);
	    //		vX = nearPoint[0];
	    //		vY = nearPoint[1];
	    //		Selection.set(id);
	}
	    break;
	case VisualSearchArea.MODE_ADDRESS:{
	    String address = space[nearPoint[0]][nearPoint[1]].getId();
	    setSelectedAddress(address);
	}
	    break;
	default:
	    break;
	}
	// System.err.println("Released");
    }

    /**
     * Describe <code>mouseDragged</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseDragged(MouseEvent e) {
    }

    /**
     * Describe <code>mouseMoved</code> method here.
     *
     * @param e a <code>MouseEvent</code> value
     */
    public void mouseMoved(MouseEvent e) {
	mouseX = e.getX();
	mouseY = e.getY();

	//	System.err.println("x: " + x + " y: " + y);

	switch (mode) {
	case VisualSearchArea.MODE_MSGID:   {
	    /*
	      if (oldX > -1 && oldY > -1) 
		space[oldX][oldY].setState(oldState);	     

	    if (space[mouseX][mouseY] != null) {
		System.err.println("HIT!");
		oldX = mouseX;
		oldY = mouseY;
		oldState = space[mouseX][mouseY].getState();
		space[mouseX][mouseY].setState(NodeState.PASSED);
		getParent().repaint();
	    }
	    */
	    break;
	}
	    
	case VisualSearchArea.MODE_ADDRESS: {
	    int nearPoint[] = isNear();
	    if (nearPoint == null) { 
		clearString();
		break;
	    }

	    AddressNodeState ans = (AddressNodeState)space[nearPoint[0]][nearPoint[1]];
	    Graphics g = getGraphics();
	    String tmp = ans.getId();

	    if (lastDrawAddress != null && tmp.equals(lastDrawAddress)) break;
	    if (lastDrawAddress != null) clearString();
	    
	    lastDrawAddress = tmp;
	    lastDrawAddressPos = new Pos(nearPoint[0], nearPoint[1]);
	    g.drawString(lastDrawAddress, nearPoint[0]/2, nearPoint[1]/2);
	}
	    break;
	    
	default:
	    break;
	}
    }

    /**
     * Describe <code>clearString</code> method here.
     *
     */
    private void clearString() {
	if (lastDrawAddress == null) return;
	
	Graphics g = getGraphics();
	g.setColor(getBackground());
	g.drawString(lastDrawAddress, 
		     lastDrawAddressPos.getX(), lastDrawAddressPos.getY());
	lastDrawAddress = null;
	lastDrawAddressPos = null;

	repaint();
    }

    /**
     * Describe <code>drawBoundingBox</code> method here.
     *
     * @param g a <code>Graphics</code> value
     */
    private void drawBoundingBox(Graphics g) {
	g.setColor(Color.black);
	g.drawLine(0,0,WIDTH,0);
	g.drawLine(WIDTH,0,WIDTH,HEIGHT);
	g.drawLine(WIDTH,HEIGHT,0,HEIGHT);
	g.drawLine(WIDTH,0,0,0);
    }

    /**
     * Add node to 
     *
     * @param orders[] an <code>int</code> value
     */
    public void addNode(int orders[], String id) {
	int x = 0;
	int y = 0;
	int ptr = 0;
	for (double i = 0; i < LEVELS; i++) {
	    int o = orders[ptr];
	    int p = 0;
	    if (o < 9) {
		p = positions[o];
		ptr++;
	    } else {
		p = 4;
		for (int j = ptr; j < orders.length; j++ ) {
		    // orders[ptr]-=8;		 
		    orders[j]-=8;		 
		} // end of for (int j = ptr; j < orders.length; j++) )
	    } // end of else

	    x += ((int)Math.pow(3d,LEVELS - i -1)) * (p%3);
	    y += ((int)Math.pow(3d,LEVELS - i -1)) * (p/3);
	    //	    System.err.println("x: " +x +", y: " + y + ", p: " + p );
	} // end of for (int i = 0; i < LEVELS; i++)

	//	System.err.println("x: " +x +", y: " + y);
	    
	//	System.err.println("----");
	if (x >= 0 && x<AREA_WIDTH && y>=0 && y<AREA_HEIGHT) {
	    space[x][y] = new NodeState(id, NodeState.MARKED);
	} // end of if (x < 0&&x<729&&y>0&&y<729)
    }

    /**
     * Describe <code>addNode</code> method here.
     *
     * @param x an <code>int</code> value
     * @param y an <code>int</code> value
     * @param addr a <code>String</code> value
     */
    public void addNode(String from, HashMap linkInfoMap) {
	// HashMap distanceMap = new HashMap();
	HashMap liMap = new HashMap();

	Set keySet = linkInfoMap.keySet();
	Iterator ite = keySet.iterator();
	while (ite.hasNext()) {
	    String localFrom = (String)ite.next();
	    LinkInfo li = (LinkInfo)linkInfoMap.get(localFrom);

	    liMap.put(localFrom, li);
	    Iterator addrs = li.getAddresses();
	    HashMap localMap = (HashMap)distanceMap.get(localFrom);
	    if (localMap == null) {
		localMap = new HashMap();
		distanceMap.put(localFrom, localMap);
	    } // end of if (localMap == null)
	    
	    while (addrs.hasNext()) {
		String addr = (String)addrs.next();
		if (localFrom.equals(addr)) continue;
		
		int cnt = li.getCount(addr);

		Integer count = (Integer)localMap.get(addr);
		if (count == null) 
		    count = new Integer(cnt);
		else 
		    count = new Integer(count.intValue()+cnt);
		localMap.put(addr, count);

		{
		    HashMap lp2 = (HashMap)distanceMap.get(addr);
		    if (lp2 == null) {
			lp2 = new HashMap();
			distanceMap.put(addr, lp2);
		    } // end of if (lp2 == null)
		    
		    Integer cnt2 = (Integer)lp2.get(localFrom);
		    if (cnt2 == null) {
			cnt2 = new Integer(cnt);
		    } // end of if (cnts == null)
		    else {
			cnt2 = new Integer(cnt2.intValue() + cnt);
		    } // end of else
		    lp2.put(localFrom, cnt2);
		}
	    } // end of while (addr.hasNext())
	} // end of while (ite.hasNext())

	{
	    int sum =0;
	    int n = 0;
	    int min = 10000;
	    int max = -1;
	    Iterator ite1 = distanceMap.keySet().iterator();
	    while (ite1.hasNext()) {
		HashMap localMap = (HashMap)distanceMap.get(ite1.next());
		Iterator ite2 = localMap.keySet().iterator();
		while (ite2.hasNext()) {
		    Integer i = (Integer)localMap.get(ite2.next());
		    sum += i.intValue();
		    n++;
		    if (i.intValue()>max) max = i.intValue();
		    if (i.intValue()<min) min = i.intValue();
		}		
	    } // end of while (ite1.hasNext())

	    cntAve = sum/n;
	    cntMax = max;
	    cntMin = min;
	}

	HashMap localDist = (HashMap)distanceMap.get(from);
	keySet = localDist.keySet();
	if (keySet == null) return;
	
	ite = keySet.iterator();

	Vector v = new Vector();
	while (ite.hasNext()) {
	    String to = (String)ite.next();
	    Integer n = (Integer)localDist.get(to);
	    v.add(new CountByAddress(to,n.intValue()));
	} // end of while (ite.hasNext())

	int len = v.size();
	CountByAddress[] counts = new CountByAddress[len];
	for (int i = 0; i<len; i++) {
	    counts[i] = (CountByAddress)v.get(i);
	} // end of for (int i = 0; i<n; i++)
	
	//	counts = (CountByAddress[])(v.toArray());

	Arrays.sort(counts, new CountByAddress());
	
	double r = 150.0d;
	double angle = 2.0d * Math.PI / (double)counts.length;

	// System.err.println("length = " + counts.length);

	int centerX = AREA_WIDTH/2;
	int centerY = AREA_HEIGHT/2;

	space[centerX][centerY] = new AddressNodeState(from, NodeState.MARKED, (LinkInfo)liMap.get(from),0.0d);
	position.put(from, new Pos(centerX,centerY));

	int cntLimit = (cntMax-cntMin)/5;
	for (int i = 0;i<counts.length; i++) {
	    double rd = r;
	    int dist = getDistance(from, counts[i].getAddress());
	    // System.err.println("Dist:"+dist);
	    if (dist > cntLimit+cntAve) {
		rd *= 0.5d;
	    } // end of if (getDistance(from, counts[i].getAddress()))
	    else if (dist <  cntAve - cntLimit ){
		rd *= 1.5d;
	    } // end of else
	
	    int x = centerX + (int)(rd * Math.sin(angle*(double)i));
	    int y = centerY - (int)(rd * Math.cos(angle*(double)i));
	    if (x >= 0 && x<AREA_WIDTH && y>=0 && y<AREA_HEIGHT) {	
		// System.err.println("x=" + x + ",y=" + y + ",addr:" +counts[i].getAddress());
		if (from.equals(counts[i].getAddress()))  continue;

		space[x][y] = new AddressNodeState(counts[i].getAddress(), NodeState.MARKED, (LinkInfo)liMap.get(counts[i].getAddress()), angle*(double)i);

		position.put(counts[i].getAddress(), new Pos(x,y));
	    }
	}

	r = 2.0d * r;
	int remain = linkInfoMap.size() - counts.length;
	if (remain == 0) return;
	
	angle = 2.0d * Math.PI / (double)remain;
	int i = 0;
	ite = linkInfoMap.keySet().iterator();

	HashMap placeCount = new HashMap();

	while (ite.hasNext()) {
	    String localFrom = (String)ite.next();
	    LinkInfo li = (LinkInfo)linkInfoMap.get(localFrom);

	    // if (position.get(localFrom) != null) continue;

	    Iterator addrs = li.getAddresses();
	    while (addrs.hasNext()) {
		String address = (String)addrs.next();

		// System.err.print("address:"+address);
		// System.err.println(",localFrom:"+localFrom);
		if (position.get(address)==null) {
		    setPlace(localFrom, address, liMap, placeCount);
		} else {
		    if (position.get(localFrom)!=null) continue;

		    setPlace(address, localFrom, liMap, placeCount);
		} // end of else
	    i++;
	    } // end of while (addrs.hasNext())
	} // end of while (ite.hasNext())
    }

    /**
     * Describe <code>getDistance</code> method here.
     *
     * @param addr1 a <code>String</code> value
     * @param addr2 a <code>String</code> value
     * @return an <code>int</code> value
     */
    private int getDistance(String addr1, String addr2) {
	try {
	    HashMap local = (HashMap)distanceMap.get(addr1);
	    Integer dist = (Integer)local.get(addr2);
	    return dist.intValue();
	} catch (NullPointerException e) {
	    return cntMax;
	} // end of try-catch
    }

    /**
     * Describe <code>setPlace</code> method here.
     *
     * @param addr1 a <code>String</code> value
     * @param addr2 a <code>String</code> value
     */
    private void setPlace(String addr1, String addr2, HashMap liMap,
			  HashMap placeCount) {

	// System.err.println("setPlace:"+addr2);
	double r = 150.0d;
	Pos pos1 = (Pos)position.get(addr1);
	AddressNodeState ans1 = (AddressNodeState)space[pos1.getX()][pos1.getY()];

	int n = ((HashMap)distanceMap.get(addr1)).size();

	// System.err.println("N:"+n);
	double dAngle = Math.PI/(3.0d*(double)n);

	double baseAngle = ans1.getAngle() - Math.PI/(3.0d * 2.0d);

	Integer cnt = (Integer)placeCount.get(addr1);
	if (cnt == null) {
	    cnt = new Integer(0);
	} // end of if (cnt == null)

	int i = cnt.intValue();
	cnt = new Integer(cnt.intValue()+1);
	placeCount.put(addr1, cnt);
	
	int x = pos1.getX() + (int)(r * Math.sin(baseAngle+dAngle*(double)i));
	int y = pos1.getY() - (int)(r * Math.cos(baseAngle+dAngle*(double)i));

	// System.err.println("Pre X:"+x+",Y="+y);

	if (x >= 0 && x<AREA_WIDTH && y>=0 && y<AREA_HEIGHT) {	
	    // System.err.println("X=" + x + ",Y=" + y + ",addr:" + addr2);
	    space[x][y] = new AddressNodeState(addr2, NodeState.MARKED, (LinkInfo)liMap.get(addr2), baseAngle+dAngle*(double)i);
		    
	    position.put(addr2, new Pos(x,y));
	}
    }

    /**
     * Describe <code>clearSpace</code> method here.
     *
     */
    public void clearSpace() {
	space = null;
	space = new NodeState[AREA_WIDTH][AREA_HEIGHT];

	position = null;
	position = new HashMap();

	selectedAddress = null;
	distanceMap = new HashMap();

	Graphics g = getGraphics();
	g.setColor(getBackground());
	g.fillRect(0,0,AREA_WIDTH,AREA_HEIGHT);
    }

    /**
     * Describe <code>setState</code> method here.
     *
     * @param msgId a <code>String</code> value
     * @param state an <code>int</code> value
     */
    public void setState(String msgId, int state) {
	ALLEXIT: 
	for (int iX = 0; iX < AREA_WIDTH; iX++) {
	    for (int iY = 0; iY < AREA_HEIGHT; iY++) {
		if (space[iX][iY] == null) continue;
		
		if (msgId.equals(space[iX][iY].getId())) {
		    space[iX][iY].setState(state);
		    break ALLEXIT;
		} // end of if ()
	    } // end of for (int iY = 0; iY < AREA_HEIGHT; iY++)
	} // end of for (int iX = 0; iX < AREA_WIDTH; iX++))
    
    }

    /**
     * Describe <code>selectionChanged</code> method here.
     *
     * @param msgId a <code>String</code> value
     */
    public void selectionChanged(String msgId) {
	boolean changed = false;
	switch (mode) {
	case VisualSearchArea.MODE_MSGID:
	    if (selectedX > -1 && selectedY > -1) {
		// Ρɤξ֤򸵤᤹
		if (space[selectedX][selectedY] != null) 
		    space[selectedX][selectedY].setState(selectedState);

		selectedX = -1;
		selectedY = -1;
		selectedState = NodeState.NONE;

		changed = true;
	    } // end of if (selectedX > -1 && selectedY > -1)

	    if (msgId != null) {
		if (mouseX>-1&&mouseY>-1) {
		    selectedX = vX;
		    selectedY = vY;
		    selectedState = space[selectedX][selectedY].getState();
		    space[selectedX][selectedY].setState(NodeState.SELECTED);
		    changed=true;
		} else {
		    ALLEXIT:
		    for (int localX = 0; localX < AREA_WIDTH; localX++) {
			for (int localY = 0; localY < AREA_HEIGHT; localY++) {
			    if (space[localX][localY] == null) continue;
			
			    if (space[localX][localY].getId().equals(msgId)) {
				mouseX = localX/2;
				mouseY = localY/2;
				selectedX = localX;
				selectedY = localY;
				selectedState = space[localX][localY].getState();
				space[localX][localY].setState(NodeState.SELECTED);
			    changed=true;
			    break ALLEXIT;
			} // end of if (space[localX][localY].getId().equals(msgId))
			
			
		    } // end of for (int localY = 0; localY < AREA_HEIGHT; localY++)
		    
		} // end of for (int localX = 0; localX < AREA_WIDTH; localX++)
		
	    } // end of else
	    } // end of if (msgId != null)
	    
	    break;
	    
	default:
	    
	    break;
	} // end of switch (mode)

	if (changed) {
	    // getParent().invalidate();
	    invalidate();
	    validate();
	    // getParent().validate();
	    repaint();
	}
    }

    /**
     * Describe <code>isNear</code> ޥݥȤζ᤯˥ݥȤ뤫Ƚꤹ롣
     *
     * @return an <code>int[]</code> ᤯˥ݥȤ {dx,dy}̵֤Сnull֤
     * 
     */
    private int[] isNear() {
	// ݥ󥿤ζ᤯õ

	int doubleX = 2 * mouseX;
	int doubleY = 2 * mouseY;

	final int DOUBLE_WIDTH = 2* BOXWIDTH ;
	final int DOUBLE_HEIGHT = 2*BOXHEIGHT;

	final int HALF_WIDTH = BOXWIDTH/2 ;
	final int HALF_HEIGHT = BOXHEIGHT/2;
	
	for (int iY = 0; iY < DOUBLE_HEIGHT; iY++) {
	    for (int iX = 0; iX < DOUBLE_WIDTH; iX++) {

		int localX = doubleX + iX - BOXWIDTH;
		int localY = doubleY + iY - BOXHEIGHT;

		// ꡼γʤ̵뤹롣
		if (localX < 0 || localY < 0) continue;
		if (localX >= AREA_WIDTH  || localY >= AREA_HEIGHT) continue;

		if (space[localX][localY] == null) continue;

		mouseX = localX/2;
		mouseY = localY/2;
		return new int[]{localX, localY};
	    
	    } // end of for (int i = 0; i < checkPoint.length; i++)
	}
	return null;
    }

    /**
     * Describe <code>setSelectedAddress</code> method here.
     *
     * @param address a <code>String</code> value
     */
    private void setSelectedAddress(String address) {

	if (selectedAddress != null) {
	    Pos pos = (Pos)position.get(selectedAddress);
	    space[pos.getX()][pos.getY()].setState(NodeState.MARKED);
	} // end of if (selectedAddress != null))
    
	selectedAddress = address;

	if (address != null) {
	    Pos pos = (Pos)position.get(address);
	    space[pos.getX()][pos.getY()].setState(NodeState.SELECTED);	    
	} // end of if (address != null))

	repaint();
    }
    /**
     * Describe <code>getSelectedAddress</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getSelectedAddress() {
	return selectedAddress;
    }

    // display area data

    private int mode = VisualSearchArea.MODE_MSGID;

    private NodeState[][] space = new NodeState[AREA_WIDTH][AREA_HEIGHT];

    private MailPopup mpu = null;

    private int mouseX = -1;
    private int mouseY = -1;

    private int vX = -1;
    private int vY = -1;

    private int oldX = -1;
    private int oldY = -1;
    private int oldState = NodeState.NONE;

    private int selectedX = -1;
    private int selectedY = -1;
    private int selectedState = NodeState.NONE;

    private String selectedAddress = null;
    private HashMap distanceMap = null;

    private int cntAve = 0;
    private int cntMin = 0;
    private int cntMax = 0;

    private String lastDrawAddress = null;
    private Pos lastDrawAddressPos = null;

    private HashMap position = null;
}

