/*
  (c) 2002, Jun Kurabe and Hiroshi Ichiji. All rights reserved.
 */
// Created by Jun Kurabe
// 2001/09/16

//package jp.co.lmer.mail;

import java.util.HashMap;
import java.util.Vector;

import EDU.auburn.VGJ.graph.Node;
import EDU.auburn.VGJ.util.*;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Address;
import javax.mail.internet.MimeUtility;
import javax.mail.internet.ParseException;
import java.awt.*;
import java.awt.Graphics.*;
import java.awt.Color;

import MailGraph;
import MailDraw;
import MailFieldConvert;
import Preface;
import Reference;

/**
 * @author Jun Kurabe
 */
public class MailGraphDrawing 
    implements MailDraw {

    /**
     */
    public MailGraphDrawing() {
	this.graph = new MailGraph();
	this.gw = new MailGraphWindow(this.graph);
    }

    /**
     */
    public void destroy() {
	gw.setVisible(false);
	gw = null;
	graph = null;
    }

    /**
     * Clear HashTable and Node on Graph
     */
    public void clear() {
	graph.clear();
	passed = new HashMap();
	color = new HashMap();
	colorMgr = new ColorManager();
    }

    /**
     * @param msgId
     * @param ref
     */
    public void draw(String msgId, Reference ref) {
	class DMG 
	    extends RIImpl {
	    public DMG(String msgId, Reference ref, MailGraphDrawing mgd) {
		this.ref = ref;
		this.mgd = mgd;
		passed.put(msgId, msgId);
	    }

	    /** */
	    public boolean followDoIt(String orgId, String childId) {
		try {
		    if ( passed.get(childId) == null ) {
			mgd.addNode(childId, ref.getMsg(childId));
			passed.put(childId, childId);
		    }
		    mgd.addEdge(orgId, childId);
		    System.err.println(orgId + "-->" + childId);
		} catch (MailFileException e) {
		    e.printStackTrace();
		}
		return false;
	    }

	    /**  */
	    private HashMap passed = null;
	    /** */
	    private Reference ref = null;
	    /** */
	    private MailGraphDrawing mgd = null;
	}

	String top = ref.draw(msgId, this);
	gw.draw(top);
    }

    /**
     * Show the Window
     */
    public void show() {
	gw.show();
    }
    
    /**
     * @param id
     * @param msg
     */
    public void addNode(String id, Message msg) {
	//	if ( passed.get(id) != null) return;
	int fId = graph.insertNode();
	Node node = graph.getNodeFromIndex(fId);
	node.setBoundingBox(300.0d, 40.0d);
	node.setShape(Node.RECTANGLE);
	//	node.setLabel(id);

	try {
	    String label = MailFieldConvert.convertSubject(msg);
	    String aFrom = MailFieldConvert.convertFrom(msg);

	    int ip = aFrom.indexOf("<");
	    String aFrom1 = aFrom.substring(0,ip-1);
	    String aFrom2 = aFrom.substring(ip);
	    aFrom = aFrom1 + "\\n" + aFrom2;
	    Color aColor = (Color)color.get(aFrom);
	    if (aColor == null ) {
		//		aColor = Color.pink;
		aColor = colorMgr.getColor();
		color.put(aFrom, aColor);
	    }
	    node.setColor(aColor);
	    node.setLabel(label+"\\n"+aFrom);
	    node.setLabelPosition("CENTER");
	    Graphics gr = gw.getGraphics();
	    if ( gr == null ) 
		node.setBoundingBox(24.0d * label.length(), 26.0d);
	    else {
		FontMetrics fm = gr.getFontMetrics();
		int wl = fm.stringWidth(label);
		int wf1 = fm.stringWidth(aFrom1);
		int wf2 = fm.stringWidth(aFrom2);
		if ( wf1 > wl ) wl = wf1;
		if ( wf2 > wl ) wl = wf2;
		node.setBoundingBox(wl, fm.getHeight()*3.0d); // 3ԤʤΤǡ30Ǥ
		//	    node.setBoundingBox(getStringWidth(label), 26.0d);
	    }
	    node.data_.put("Content",MailFieldConvert.getBodyText(msg));
	    node.data_.put("MsgId",msg.getHeader("Message-ID")[0]);
	} catch (MessagingException me) {
	    me.printStackTrace();
	} catch (MailFileException me) {
	    me.printStackTrace();
	}

	if (Preface.debug) System.err.println("id =" + id);
	passed.put(id, new Integer(fId));
    }

    /** */
    static private final double as = 8.0d;
    /** */
    static private final double ds = 16.0d;

    /**
     * Add Edge between nodes
     * @param formId
     * @param toId
     */
    public void addEdge(String fromId, String toId) {
	Integer fId = (Integer)passed.get(fromId);
	Integer tId = (Integer)passed.get(toId);

	if (fId == null || tId == null ) return;

	graph.insertEdge(fId.intValue(), tId.intValue());
	if (Preface.debug) System.err.println("fromId =" + fromId + " toId = " + toId);
    }

    /**
     * Add Edge between mails
     * @param graph
     * @param nodes
     * @param from
     * @param to
     */
    public void addEdge(MailGraph graph, Nodes nodes, int from, int to) {
	int fromId = nodes.getFromIdx(from);
	int toId = nodes.getFromIdx(to);

	graph.insertEdge(fromId, toId);
    }

    public String getSelectedMsgId() {
	return gw.getSelectedMsgId();
    }

    /** */
    private MailGraph graph = null;
    /** */
    private MailGraphWindow gw = null;
    /** */
    private HashMap passed = null;

    /** */
    private HashMap color = null;
    /** */
    private ColorManager colorMgr = null;
}
	
/** */
class Nodes 
    extends Vector {
    /** */
    public Nodes () {
	super();
    };

    /** 
     * @param value
     */
    public void addElement(int value) {
	super.addElement(new Integer(value));
    }

    /** 
     * @param idx
     * @return int
     */
    public int getFromIdx(int idx) {
	return ((Integer)super.elementAt(idx)).intValue();
    }
}

/**
 * 2002/01/13
 */
class ColorManager {
    
    /**
     */
    public ColorManager() {
    }

    /**
     * return a Color
     * @return Color 
     */
    public Color getColor() {
	Color rv = colors[index++];
	if ( index >= colors.length) index = 0;
	return rv;
    }

    /** */
    private int index = 0;

    /** */
    static private Color[] colors = {Color.pink, 
				     Color.lightGray, 
				     Color.cyan, 
				     Color.yellow, 
				     Color.green,
				     Color.orange};
}
